#ifndef LIGHTS_H
#define LIGHTS_H

#include <GLM/glm.hpp>

#include "Model.h"

// Максимальное число источников света
#define MAX_LIGHTS 300

// Точечный источник света
struct LightData 
{
    alignas(16) glm::vec3 position; // Позиция
    alignas(16) glm::vec3 color; // Цвет 
};

// Источник света
class Light : public Node
{
    public:
        static int getUBOsize(); // Возвращает размер буфера в байтах
        static void upload(UBO& lights_data); // Загрузка данных в буфер

        static Light& getNew(); // Возвращает ссылку на новый источник света
        void destroy(); // Уничтожает источник света

        const glm::vec3& c_color() const; // Константный доступ к цвету
        glm::vec3& e_color(); // Неконстантная ссылка для изменений цвета

        static void render(ShaderProgram &shaderProgram, UBO &material_buffer); // Рисование отладочных лампочек
    private:
        Light(); // Конструктор без параметров
        Light(const Light& copy) = delete; // Конструктор копирования ОТКЛЮЧЕН
        Light& operator=(const Light& other); // Оператор  присваивания
        virtual ~Light(); 

        glm::vec3 color; // Цвет

        int index; // Индекс в массиве отправки (может не совпадать с lights) для дефрагментированного доступа
        static Light& findByIndex(GLuint index); // Возвращает ссылку на источник с нужным индексом

        bool uploadReq; // Необходимость загрузки в следствии изменений
        void check_id(); // Проверка что не взаимодествуем с пустым источником
        void toData(); // Преобразует информацию об источнике в структуру LightData

        virtual void recalcMatrices(); // Метод пересчета матрицы трансформации по необходимости, должен сбрасывать флаг changed

        static GLuint count; // количество используемых источников (должно быть <= MAX_LIGHTS)
        static LightData data[MAX_LIGHTS]; // Массив данных по источникам света
        static Light lights[MAX_LIGHTS]; // Массив источников-узлов сцены
};

#endif // LIGHTS_H
